/* ---------------------------------------------------------------------- */
SUBSCHEMA Decode AUTHOR 'Wolfgang Wander & Alexander Kisselev'
/* ---------------------------------------------------------------------- */

DEFINE ESET
  dataProp = (
    iWire       = INTE[1,*]: 'strip or wire no. which fired',
    rWirePos    = REAL     : 'position of wire/strip after alignment',
    iBitMap     = BITP     : 'cluster bit map; CAUTION: use 30 bits only'
  ): 'Proportional Chamber or Vertex Chamber';

  dataPuls = (
    iWire     = INTE[1,*]: 'strip or wire no. which fired',
    iPuls     = INTE     : 'TDC value/ADC value',
    rWirePos  = REAL     : 'position of wire/strip after alignment',
    rPuls     = REAL     : 'drift distance/puls height'
  ): 'FC,BC,Cer,TRD';

  dataHodo = (
    iWire     = INTE[1,*]: 'strip no. which fired',
    iADC      = INTE     : 'ADC value',
    iTDC      = INTE     : 'TDC value',
    rPuls     = REAL     : 'drift distance',
    rTime     = REAL     : 'converted TDC time'
  ): 'Hodoscopes';

  mapdataCalo = (
    iADC(420)  = INTE : 'ADC value for all 420 cells'
  ): 'Calorimeter';

  mapdataLumi = (
    iADC(24)  = INTE : 'ADC value for all 24 cells'
  ): 'Lumi Monitor';

  dataCalo = (
    iCell     = INTE[1,420]:      'Cell of Calorimeter or Lumi',
    rPuls     = REAL       :      'Energy Deposit in Cell'
  ): 'Lumi Monitor / Calorimeter';

  dataCaloSums = (
    iColumn   = INTE[1,*]: 'Calorimeter Column#',
    iADC      = INTE     : 'ADC value',
    iTDC      = INTE     : 'TDC value',
    rPuls     = REAL     : 'pulse height',
    rTime     = REAL     : 'converted TDC time'
  ): 'Calorimeter sums';
END ESET

DEFINE RSET
  ( dataProp    [1,1] -> dgDETS[0,*] );
  ( dataPuls    [1,1] -> dgDETS[0,*] );
  ( dataHodo    [1,1] -> dgDETS[0,*] );
  ( dataCalo    [1,1] -> dgDETS[0,*] );
  ( dataCaloSums[1,1] -> dgDETS[0,*] );
END RSET

DEFINE ESET
  dataEvKey = ( 
    iDummy1   = INTE [1,*] : 'First integer in the ADAMO WCB', 
    iDummy2   = INTE [1,*] : 'Second one to be consistend with HRC.ddl',
    cType    = CH32        : 'Selection type'
  ) : 'Key for Decoder Dataflow';
END ESET

DEFINE ESET
  dcEvInfo = (
    iRun           = INTE [1,*] : 'Run Number',
    iEvent         = INTE [1,*] : 'Event Number',

      /* Precision of this time is roughly 10 sec; */
    iEvUnixTime    = INTE : 'Event UNIX time, taken from the header',

      /*  0 - Ok;                                             */
      /* -1 - Syncing was off;                                */
      /* -2 - Event belongs to the leading (incomplete) burst;*/
      /* -3 - Event belongs to the trailing (unsynced) burst; */
      /* -4 - VME timer error;                                */ 
    iSyncFlag      = INTE : 'Event synchronization flag',

      /* Valid if only iSyncFlag = 0 (or -2?); */
    iEffEvUnixTime = INTE : 'Effective UNIX time (corrected)',

      /* Intervals are very presice, but big relative shifts happen; */
      /* if can not be converted, set to '-1';                      */
    iEvUnixTimeVME = INTE : 'Event UNIX time, taken from the VME module',

      /* iMicroTimeVME>=0 are additional microseconds from the same */
      /* module; iMicroTimeVME may be -1, if misroseconds there are */
      /* wrong (really happens!);                                   */
    iMicroTimeVME  = INTE : 'Additional microseconds from the VME module',

      /* The same for the 2-d VME module (valid from Run 14960);    */
      /* if can not be converted, set to '-1';                      */
    iEvUnixTimeVME1 = INTE : 'Event UNIX time, taken from the VME module',
      /* iMicroTimeVME>=0 are additional microseconds from the same */
      /* module; iMicroTimeVME may be -1, if misroseconds there are */
      /* wrong (really happens!);                                   */
    iMicroTimeVME1  = INTE : 'Additional microseconds from the VME module',

      /* Due to some DAQ problems the VME time can jump backwards in time */
      /* these jumps are corrected by a two pass running of hdc over the  */
      /* raw data. Jump corrections are stored here as                    */
      /* corrected_time - original_time                                   */
      /* whenever the VME time module returned only zeros this variable   */
      /* is set to -999.                                                  */
    rJumpCorr      = REAL: 'VME Time jump correction',

      /* Status bit pattern of the VME time modules:                      */
      /* 0x1:  first  VME time module set to zero                         */
      /* 0x2:  second VME time module set to zero                         */
      /* 0x4:  time from first  VME time corrected                        */
      /* 0x8:  time from second VME time corrected (not yet implemented)  */
    bStatusTimeVME = BITP : 'Status bit pattern for the VME time modules',

      /* dRevCounter = double(iRevMS)*100000000 + double(iRevLS); */
    iRevMS         = INTE : 'HERA revolution counter - MS part',
    iRevLS         = INTE : 'HERA revolution counter - LS part',
    iBunch         = INTE : 'HERA bunch number',
        
    polStatus      = BITP : 'polarization status register(16 bits)',
    polADC(4)      = INTE : 'polarization ADCs',
    daqFlags(3)    = BITP : 'DAQ flags ##29-31',

    iTriggerMask   = BITP : 'Trigger Mask',
    iReadoutMask   = BITP : 'Readout Mask',
    iBurstNum      = INTE : 'Burst Number',
    iEffBurstNum   = INTE : 'Effective Burst Number',
    iLocalEvNum    = INTE : 'Event number within burst',

    iLatchTRD(4)   = INTE : 'TRD Latch data',

    rPhaseMon      = REAL : 'Phase Monitor data',
    rPhaseMonDig   = REAL : 'Phase Monitor data (digital device)',
    iGmsMon        = INTE : 'GMS filter wheel position signal',

    iHVStatus      = BITP : 'High Voltage status bits',

    iHeraClock(2)  = INTE : '2 HERA Clock TDCs',
    iPickUp        = INTE : 'Pick up TDC'
  ) : 'Additional Event Info';
END ESET

DEFINE ESET
  dcGMSInfo = (
    iRefDetPin(3) = INTE : 'Photodiode reference detector data'
  ) : 'Part of GMS specific info';
END ESET

DEFINE ESET
  dcTestBC = (
    bcAnalog(2)     = INTE : 'BC Test data',
    bcTestNdet      = INTE : 'Ask Lachnit, please',
    bcTestNplane    = INTE : '...................',
    bcTestMvolt     = INTE : '...................',
    bcTestMverr     = INTE : '...................',
    bcDrift(14)     = INTE : '...................'
  ) : 'BC Test data';
END ESET


DEFINE ESET
  dcTrigger = (
    iTrgId   = INTE[1,96] : 'Trigger channel number',
    iTDCval  = INTE       : 'TDC value',
    cTrgType = CHA4       : 'Trigger Type'
  ) :'Event Trigger Information';
END ESET

DEFINE DATAFLOW
  dataDfl = {dataProp}, {dataPuls}, {dataHodo}, {mapdataCalo},
    {mapdataLumi}, {dataCalo}, {dataCaloSums}, {dcEvInfo}, {dcTrigger},
    {dcTestBC}, {dcGMSInfo} : 'Decoder Output DataFlow';
END DATAFLOW

END SUBSCHEMA

