SUBSCHEMA Recons AUTHOR 'Wolfgang Wander'

DEFINE ESET
rcEvKey =( iEvent = INTE [1,*] : 'Event Number',
            iRun   = INTE [1,*] : 'Run Number',
            cType    = CH32       : 'Selection type')
            : 'Key for Reconstructed Events';


rcEvInfo =
  (
  iMaxMulFront = INTE :'Front Chamber Maxmul for this event',
  iMaxMulBack  = INTE :'Back Chamber Maxmul for this event',
  iHeap        = INTE :'Heap Requirements for this event',
  rTime        = REAL :'Seconds used for this event',
  iTreelines   = INTE :'Number of Generated Treelines',
  iPartTracks  = INTE :'Number of Generated Partial Tracks'
  ) :'A growing collection of hrc event statistics';

rcRaw =
  (
/* --- VCs: Organized as VC(1): | u-wire VC1 | u-wire VC2 | (hi|low)
                         VC(2): | v-wire VC1 | v-wire VC2 |
                         VC(3): | x-wire VC1 | x-wire VC2 |  ---
       definition of the 16 bit bitpattern (a little chaotic):
  
   x-plane:
         0-9   : wirenumber  0-682 possible  ( needs 10 bits ! )
          10   : marks a cluster with 2-3 wires
       13-14   : determines the x plane more precise 
   uv-plane:
        0-11   : wirenumber  0-2170 possible  ( needs 12 bits ! )
          13   : marks a cluster with 2-3 wires
   all:
          12   : marks a cluster with an even number of wires 
          15   : 0 good for calibration - 1 not good for calibration    */
  VC(3)    = BITP : 'Vertex Chamber Raw Data',

/* --- DVCs: Organized as DVC(i): | wire | TDC |
       and i counting from DVC1 u,u',v,v',x,x' 
    where
      corr is a 8bit calibration/tracking correction of type
        signed char. The value is measured in 20 um. +127 or -128 incicate
        an overflow.
        The resulting number is the difference between the
          straight track position and an optimal hit.
      wire is a 8bit wire number of type unsigned char
      TDC is a 16bit TDC value of type unsigned short.         --- */

  DVC(6)   = BITP : 'Drift Vertex Chamber Raw Data',

/* --- FCs: Organized as FC(i): | wire | TDC |
       and i counting from FC1 u,u',v,v',x,x' - FC2 u,u' ... 
    where
      corr is a 8bit calibration/tracking correction of type
        signed char. The value is measured in 20 um. +127 or -128 incicate
        an overflow.
        The resulting number is the difference between the
          straight track position and an optimal hit.
      wire is a 8bit wire number of type unsigned char
      TDC is a 16bit TDC value of type unsigned short.         --- */

  FC(12)   = BITP : 'Front Chamber Raw Data',

/* --- MCs: Organized as MC(1): | u-wire MC1 | v-wire MC1 |
                         MC(2): | u-wire MC2 | v-wire MC2 |
                         MC(3): | u-wire MC3 | v-wire MC3 |
                         MC(4): | x-wire MC1 | x-wire MC2 |
                         MC(5): | x-wire MC3 | rsrvd      |  --- */
  MC(5)    = BITP : 'Magnet Chamber Raw Data',

/* --- BCs: Organized as BC(i): | corr|wire | TDC |
       and i counting from BC1 u,u',v,v',x,x' - BC2 u,u' ... 
    where
      corr is a 8bit calibration/tracking correction of type
        signed char. The value is measured in 20 um. +127 or -128 incicate
        an overflow.
        The resulting number is the difference between the
          straight track position and an optimal hit.
      wire is a 8bit wire number of type unsigned char
      TDC is a 16bit TDC value of type unsigned short.         --- */

  BC(24)   = BITP : 'Back Chamber Raw Data',

/* --- Hodos: Organized as Hodo(i): | TDC | ADC | 
       and i counting from H0, H1, Pre                       --- */
  Hodo(3)  = BITP : 'Hodoscope Raw Data',

/* --- Cerenkov: Organized as CerCell: Smallest Cell Index in Square
                              Cer(1) | ADC0/0 | ADC1/0 |
                              Cer(2) | ADC0/1 | ADC1/1 |
                              Cer(3) | TDC0/0 | TDC1/0 |
                              Cer(4) | TDC0/1 | TDC1/1 |
   where x/y denotes the distance in cells from CerCell      --- */
  CerCell  = BITP : 'Cerenkov Cell',
  Cer(4)   = BITP : 'Cerenkov Raw Data',

/* --- TRD: Organized as: TRDWire(i): | Wire0 | Wire1 |
          rightmost wire of TRD1, TRD3, TRD5 for Wire0
                  resp.     TRD2, TRD4, TRD6 for Wire1
                          TRD(i)      | ADC0  | ADC1  |
                          TRDPuls(i)  | added calibrated pulses |
          The ADC Values on each TRD Cluster (two cells)     --- */
  TRDWire(3) = BITP: 'TRD Wires',
  TRD(6)     = BITP: 'TRD Raw ADC Data',
  TRDPuls(6) = REAL: 'Calibrated TRD Data',

/* --- Calo: Organized as  Calo(1): | ADC -1/-1 | ADC  0/-1 |
                           Calo(2): | ADC  1/-1 | ADC -1/ 0 |
                           Calo(3): | ADC  0/ 0 | ADC  1/ 0 |
                           Calo(4): | ADC -1/ 1 | ADC  0/ 1 |
                           Calo(5): | ADC  1/ 1 | TDC  0/ 0 |
            where x/y denotes the distance from rcCluster iX/iY --- */
  Calo(5)    = BITP: 'Calorimeter Raw Information'
  ) : 'Raw Data along Tracks';

rcVertex =
  (
  X        = REAL [ -999.000, 999.000]: 'Vertex X Position',
  Y        = REAL [ -999.000, 999.000]: 'Vertex Y Position',
  Z        = REAL [ -999.000, 999.000]: 'Vertex Z Position',
  cMethod  = CHA4 'STD'|'NOVC'|'FITV': 'Reconstruction Method',
  Cov(6)   = REAL : 'cov(x,x),cov(x,y),cov(x,z),cov(y,y),cov(y,z),cov(z,z)',
  nTracks  = INTE [        1, *       ]: '# of tracks that form vertex'
  )
  : 'Vertex obtained by the vertex processor';

rcTreeLine =
  (
  X        = REAL [ -999.00, 999.00]: 'Track Position at Mag Center',
  XSlope   = REAL [ -2.00  , 2.00  ]: 'Track Slope',
  CovXXSl(3)  = REAL                : 'Cov(X,X), Cov(X,Xsl) and Cov(Xsl,Xsl)',
  XChi	   = REAL [ 0.00   , 100.00]: 'Chi Square for this track',
  bUsed	   = INTE [ 0	   , 1	   ]: 'Track is used for further search',
  bVoid	   = INTE [ 0	   , 1	   ]: 'Track is marked void',
  NumHits  = INTE [ 0	   , 30	   ]: 'Number of correlated hits',
  cDir     = CHA4 'UDIR'|'VDIR'|'XDIR': 'direction',
  cMethod  = CHA4 'STD'|'NOVC'|'FITV' : 'Reconstruction Method',
  cPart    = CHA4 'FORW'|'BACK'       : 'detector part',
  cWhere   = CHA4 'UPP' |'LOW'        : 'detector half'
  )
: 'Track in one of the used projections (u v x)';

rcPartTrack =
  (
  X	   = REAL [ -999.00, 999.00]: 'Track Position at Mag Center',
  Y	   = REAL [ -999.00, 999.00]: 'Track Position at Mag Center',
  XSlope   = REAL [ -2.00  , 2.00  ]: 'Track Slope',
  YSlope   = REAL [ -2.00  , 2.00  ]: 'Track Slope',
  PLen     = REAL                   : 'Pathlength at z=0',
  PLenSlop = REAL                   : 'Pathlength slope to mult. with z',
  CovX(10) = REAL : 'cov(x,x), cov(x,y), cov(x,xs),cov(x,ys),cov(y,y)...',
  XYChi	   = REAL [ 0.00   , 100.00]: 'Chi Square for this track',
  ECalor   = REAL [ 0.00   , 100.00]: 'Energy in Cal Cluster',
  rclProb  = REAL [ 0.00   , 1.00  ]: 'Probability for Cl. belongs to track',
  bUsed	   = INTE [ 0	   , 1	   ]: 'Track is used for further search',
  iMissHits= INTE [ 0      , 36    ]: 'Number of missing hits (alltogether)',
  iMissVC  = INTE [ 0      , 144   ]: 'Number of missing VC Hits',
  cMethod  = CHA4 'STD'|'NOVC'|'FITV': 'Reconstruction Method',
  cPart    = CHA4 'FORW'|'BACK'       : 'detector part'
/*  cWhere   = CHA4 'UPP' |'LOW'        : 'detector half' */
  )
: 'Track in Space but in forward/backward region';

rcTrack =
  (
  ZVx	   = REAL [ -999.00, 999.00]: 'ZPosition of Vertex',
  TVx      = REAL [ -100.00, 100.00]: 'Transversal component of Vertex',
  Theta	   = REAL [    0.00,   1.00]: 'Theta = sqrt(xslope^2+yslope^2)',
  Phi	   = REAL [    0.00,   6.29]: 'Phi   = atan(y_slope/x_slope)',
  AngleCorr= INTE [       0,      1]: '1 if Theta & Phi are corrected for 
                                       holding field',
/*  SlopeDiff= REAL [   -1.00,   1.00]: 'Slope change in Magnet', */
  P	   = REAL [    0.00, 100.00]: 'Momentum determined in Magnet',
  ECalor   = REAL [    0.00, 100.00]: 'Momentum determined in Calorimeter',
  PulsTRD  = REAL [ 0.00000,  *    ]: 'Pulse Height in TRD',
  PulsH0   = REAL [ 0.00000,  *    ]: 'Pulse Height in H0',
  PulsHodo = REAL [ 0.00000,  *    ]: 'Pulse Height in Hodoscope',
  PulsPre  = REAL [ 0.00000,  *    ]: 'Pulse Height in Preshower',
  PulsCer  = REAL [ 0.00000,  *    ]: 'PhotoElectrons in Cerenkov',
  TimeH0   = REAL                   : 'Time in H0',
  TimeHodo = REAL                   : 'Time in Hodoscope',
  TimePre  = REAL                   : 'Time in Preshower',
  TimeCer  = REAL                   : 'Time in Cerenkov',
  TimeCalo = REAL                   : 'Time in Calor Sum',
  iCharge  = INTE [   -1   , 1	   ]: 'Charge',
/*  rProbElec= REAL [    0.00, 1.00  ]: 'Norm. Probability for Electron total', */
/*  rProbPi  = REAL [    0.00, 1.00  ]: 'Norm. Probability for Pion total', */
/*  rProbJunk= REAL [    0.00, 1.00  ]: 'Norm. Probability for Junk track total', */
/*  XBj	   = REAL [    0.00, 1.00  ]: 'X Bjorken', */
/*  Y	   = REAL [    0.00, 1.00  ]: 'Y',         */
/*  Q2	   = REAL [    0.00, 250.00]: 'Q^2',       */
/*  W2       = REAL [    0.00, 250.00]: 'W^2',     */
/*  Nu	   = REAL [    0.00, 100.00]: 'nu',        */
  bRecoEv  = INTE [    0   , 1	   ]: 'is generated by using mcHits',
/*  iYTracks = INTE [    0   , 20    ]: 'Number of Y-Tracks', */
  mcTrack  = INTE                   : 'Track Number in HMC',
  bUsed	   = INTE [ 0	   , 1	   ]: 'Track is used for further search',
  bInBound = INTE [ 0	   , 1	   ]: 'Lookuptable usable',
  cMethod  = CHA4 'STD'|'NOVC'|'FITV': 'Reconstruction Method'
/*  cWhere   = CHA4 'UPP' |'LOW'      : 'detector half', */
/*  GPType   = INTE                   : 'Geant Particle Type from HMC' */
  )
: 'Track in whole spectrometer';

rcTRDPuls =
  (
  iBegWire  = INTE [     0 , 10000 ]: 'Beginnig Wire of Cluster',
  iEndWire  = INTE [     0 , 10000 ]: 'End Wire of Cluster',
  rPuls     = REAL                  : 'Cluster Puls'
  )
: 'TRD Information';

rcBridge =
  (
  YSlopeMch= REAL [ -10.00 , 10.00 ]: 'Matching of Y-Slope',
  XSlopeMch= REAL [ -10.00 , 10.00 ]: 'Matching of X-Slope',
  YMch	   = REAL [ -10.00 , 10.00 ]: 'Matching of Y (incl. YOffs)',
  XMch	   = REAL [ -10.00 , 10.00 ]: 'Matching of X (incl. XOffs)',
  YOffs    = REAL [ -10.000, 10.000]: 'Calculated theoretical Offset',
  XOffs    = REAL [ -10.000, 10.000]: 'Calculated theoretical Offset',
  YSMch    = REAL                   : 'YSlope matching (incl. YSOffs)',
  YSOffs   = REAL                   : 'YSlope theoretical matching'
  )
: 'Bridging Information';

rcCluster  =
  (
  iX       = INTE [ 0	   , 100   ]: 'Cell Number of Cl.Center',
  iY	   = INTE [ 0	   , 50	   ]: 'Cell Number of Cl.Center',
  X	   = REAL [ -999.9 , 999.9 ]: 'Cluster Center',
  Y	   = REAL [ -999.9 , 999.9 ]: 'Cluster Center',
  E	   = REAL [ 0.0	   , 100.0 ]: 'Cluster Energy',
  Edelta   = REAL [ 0.0	   , 100.0 ]: 'Error in Energy',
  Time     = REAL [ 0.00000,  *    ]: 'Time from CalorSum',
  NCells   = INTE [ 0	   , 100   ]: 'Number of Cells in Cluster',
  bUsed	   = INTE [ 0	   , 1	   ]: 'Used in Reco so far',
    /* *** AYK, 08.10.98 ***                                */
    /* Pulse Height Sum over the 3 closest Preshower slabs; */
  PulsPre  = REAL [ 0.00000,  *    ]: 'Pulse Height in Preshower'
/*   cWhere   = CHA4 'UPP' |'LOW'      : 'detector half' */
  )
: 'Cluster Information';

rcLumiCl  =
  (
  iX       = INTE [ 0	   , 100   ]: 'Cell Number of Cl.Center',
  iY	   = INTE [ 0	   , 50	   ]: 'Cell Number of Cl.Center',
  X	   = REAL [ -999.9 , 999.9 ]: 'Cluster Center',
  Y	   = REAL [ -999.9 , 999.9 ]: 'Cluster Center',
  E	   = REAL [ 0.0	   , 100.0 ]: 'Cluster Energy',
  Eleak    = REAL [ 0.0    , 100.0 ]: 'Leakage'
  )
: 'Lumi Cluster Information';

rcDigitList = 
  (
   rAligcorr  = REAL		: 'Alignment correction (level III)  ',
   rCalibcorr = REAL		: 'Calibration correction (level III)',
   rTrackcorr = REAL		: 'Tracking correction (level IV)    ',
   rResultPos = REAL		: 'Resulting hit posisition          '
  ) : 'link to the data.... tables';

rcSpacePoint =
  (
  X = REAL  : 'X coordinate',
  Y = REAL  : 'Y coordinate',
  Z = REAL  : 'Z coordinate',
  dX = REAL : 'difference between X and X(U,V)'
) : 'Space Point';

rcSpaceDig  =
  ( 
  rPos = REAL : 'Hit position as used by HRC'
  ) : 'Link to Digits';

rcPIDInfo2 =
  (
/* rProbElec= REAL [    0.00, 1.00  ]: 'Norm. Probability for Electron total', */
/*  rProbPi  = REAL [    0.00, 1.00  ]: 'Norm. Probability for Pion total',    */
/*  rProbJunk= REAL [    0.00, 1.00  ]: 'Norm. Probability for Junk track total', */
/*  rpElHodo = REAL [    0.00,  *    ]: 'Prob. for Electron in Hodoscope', */
/*  rpElPres = REAL [    0.00,  *    ]: 'Prob. for Electron in Preshower', */
/*  rpElCher = REAL [    0.00,  *    ]: 'Prob. for Electron in Cherenkov', */
/*  rpElTRD  = REAL [    0.00,  *    ]: 'Prob. for Electron in TRD',       */
/*  rpElCalo = REAL [    0.00,  *    ]: 'Prob. for Electron in Calorimeter', */
/*  rpPiHodo = REAL [    0.00,  *    ]: 'Probability for Pion in Hodoscope', */
/*  rpPiPres = REAL [    0.00,  *    ]: 'Probability for Pion in Preshower', */
/*  rpPiCher = REAL [    0.00,  *    ]: 'Probability for Pion in Cherenkov', */
/*  rpPiTRD  = REAL [    0.00,  *    ]: 'Probability for Pion in TRD',       */
/*  rpPiCalo = REAL [    0.00,  *    ]: 'Probability for Pion in Calorimeter', */
/*  rErrElec = REAL [    0.00, 1.00  ]: 'Likelihood for Electron total',       */
/*  rErrPi   = REAL [    0.00, 1.00  ]: 'Likelihood for Pion total',           */
/*  reElHodo = REAL [    0.00,  *    ]: 'Likelihood for Electron in Hodoscope', */
  reElPres = REAL [    0.00,  *    ]: 'Likelihood for Electron in Preshower',
  reElCher = REAL [    0.00,  *    ]: 'Likelihood for Electron in Cherenkov', 
  reElTRD  = REAL [    0.00,  *    ]: 'Likelihood for Electron in TRD', 
  reElCalo = REAL [    0.00,  *    ]: 'Likelihood for Electron in Calorimeter',
/*  rePiHodo = REAL [    0.00,  *    ]: 'Likelihood for Pion in Hodoscope', */
  rePiPres = REAL [    0.00,  *    ]: 'Likelihood for Pion in Preshower',
  rePiCher = REAL [    0.00,  *    ]: 'Likelihood for Pion in Cherenkov', 
  rePiTRD  = REAL [    0.00,  *    ]: 'Likelihood for Pion in TRD',  
  rePiCalo = REAL [    0.00,  *    ]: 'Likelihood for Pion in Calorimeter'
  )
:  'Level 2 pid information for tracks';
END ESET

/* -------------- comment out totally
DEFINE ESET
rcPIDInfo1 =
  (
  fElec    = INTE [ 0	   , 1	   ]: 'Good electron ID in all Detectors',
  fPion    = INTE [ 0	   , 1	   ]: 'Good pion ID in all Detectors',
  fElCer    = INTE [ 0	   , 1	   ]: 'Flagged as electron in Cerenkov',
  fElPre    = INTE [ 0	   , 1	   ]: 'Flagged as electron in Preshower',
  fElCal    = INTE [ 0	   , 1	   ]: 'Flagged as electron in Calorimeter', 
  fElTRD    = INTE [ 0	   , 1	   ]: 'Flagged as electron in TRD',
  fPiCer    = INTE [ 0	   , 1	   ]: 'Flagged as pion in Cerenkov',
  fPiPre    = INTE [ 0	   , 1	   ]: 'Flagged as pion in Preshower',
  fPiCal    = INTE [ 0	   , 1	   ]: 'Flagged as pion in Calorimeter',
  fPiTRD    = INTE [ 0	   , 1	   ]: 'Flagged as pion in TRD',
  elNoCer   =  INTE [ 0	   , 1	   ]: 'Good electron ID not using Cerenkov',
  elNoPre   =  INTE [ 0	   , 1	   ]: 'Good electron ID not using Preshower',
  elNoCal   =  INTE [ 0	   , 1	   ]: 'Good electron ID not using Calorimeter',
  elNoTRD   =  INTE [ 0	   , 1	   ]: 'Good electron ID not using TRD',
  piNoCer   =  INTE [ 0	   , 1	   ]: 'Good pion ID not using Cerenkov',
  piNoPre   =  INTE [ 0	   , 1	   ]: 'Good pion ID not using Preshower',
  piNoCal   =  INTE [ 0	   , 1	   ]: 'Good pion ID not using Calorimeter',
  piNoTRD   =  INTE [ 0	   , 1	   ]: 'Good pion ID not using TRD'
  )
: 'Track in whole spectrometer (PID information)';
END ESET

*/

DEFINE RSET
(rcDigitList [1,1]  -> rcTreeLine [1,*]);
(rcDigitList [1,1]  -> dataHodo[0,*] |
                    -> dataCalo[0,*] |
		    -> dataProp[0,*] |
	            -> dataPuls[0,*] BY DigTyp );
/* (rcBridge    [1,1]  -> dataProp[0,*] BY magHit1 ); */
/* (rcBridge    [1,1]  -> dataProp[0,*] BY magHit2 ); */
/* (rcBridge    [1,1]  -> dataProp[0,*] BY magHit3 ); */
/* (rcBridge    [1,1]  -> dataProp[0,*] BY magHit4 ); */
/* (rcBridge    [1,1]  -> dataProp[0,*] BY magHit5 ); */
/* (rcBridge    [1,1]  -> dataProp[0,*] BY magHit6 ); */
/* (rcBridge    [1,1]  -> dataProp[0,*] BY magHit7 ); */
/* (rcBridge    [1,1]  -> dataProp[0,*] BY magHit8 ); */
/* (rcBridge    [1,1]  -> dataProp[0,*] BY magHit9 ); */
(rcBridge    [1,1]  -> rcSpacePoint[0,1]);
(rcSpacePoint[1,1]  -> dgDETS[0,1]);
(rcSpacePoint[1,1]  -> rcSpacePoint[0,1] BY Next );
/* (rcTRDPuls   [1,1]  -> rcTrack[0,1]); */
(rcTRDPuls   [1,1]  -> rcPartTrack[0,1]); 
(rcTRDPuls   [1,1]  -> dgDETS[0,1]);
(rcSpaceDig  [1,1]  -> rcSpacePoint[1,*]);
(rcSpaceDig  [1,1]  -> dataHodo[0,*] |
                    -> dataCalo[0,*] |
		    -> dataProp[0,*] |
	            -> dataPuls[0,*] BY DigTyp );

(rcPartTrack [1,1]  -> rcTreeLine [0,*] BY uDir) INVERTED;
(rcPartTrack [1,1]  -> rcTreeLine [0,*] BY vDir) INVERTED;
(rcPartTrack [1,1]  -> rcTreeLine [0,*] BY xDir) INVERTED;

(rcTrack [1,1]      -> rcPartTrack [0,*] BY Forward) INVERTED;
(rcTrack [1,1]	    -> rcPartTrack [0,*] BY Backward) INVERTED;
(rcTrack [0,1]      -> rcBridge    [0,*] );

(rcTrack [0,1]      -> rcVertex [0,*]);

(rcTrack [0,1]      -> rcRaw[0,1]);

(rcPartTrack[0,1]   -> rcCluster[0,*]);
/* ( rcTrack [1,1]    -> rcPIDInfo1 [1,1]); */
( rcTrack [1,1]    -> rcPIDInfo2 [1,1]);
END RSET
       
DEFINE DATAFLOW
   rEvents = {rcLumiCl},{rcCluster},{rcTrack},{rcPartTrack},{rcTreeLine},
              {rcDigitList},{rcVertex},{rcBridge},{rcSpacePoint},
	      {rcSpaceDig},/*{rcPIDInfo1},*/{rcPIDInfo2},{rcSET},{rcTRDPuls},
	      {rcEvInfo},{rcRaw}
           : 'reconstructed events information';
   rcEvents = rEvents,{dcEvInfo};
   rmEvents = mcEvents,rEvents
        : 'reconstructed events information + mcEvents';
END DATAFLOW

END SUBSCHEMA

