
!=====================================================================
! SECTION ONE: First write the user routines needed by the HANNA frame
!=====================================================================

	integer function user_args ()

	implicit none
#include "frame.inc"

! Initialize return code to success

	user_args = 0

! Create user switches

! ... filename base for ntuple files
	call addarg('-out','char*80',
     +		'filename base for output files [D: nt]','nt')

! ... control of ntuple splitting
	call addarg('-split','',
     +		'split ntuple file every nsplit events','')
	call addarg('-nsplit','integer',
     +	  'no. of records in each split ntuple [D: 20000]','20000')
	call addarg('-runsplit','',
     +	  'split ntuple file every run (supercedes -split)','')

! ... good_event and good_track criteria
	call addarg('-scat','',
     +		'only store events where scattered lepton was accep','')
	call addarg('-accep','',
     +		'only store accepted tracks','')
	call addarg('-lund','',
     +		'only store stable (cType = lund) tracks','')
	call addarg('-charge','',
     +		'only store charged tracks','')
	call addarg('-allneut','',
     +		'allow non-photon neutrals to be accepted','')

! ... acceptance definition
	call addarg('-pmin','real',
     +		'acceptance: min momentum [D: 0]','0')
	call addarg('-thmin','real',
     +		'acceptance: min theta [D: 0]','0')
	call addarg('-thmax','real',
     +		'acceptance: max theta [D: pi]','3.14159')
	call addarg('-thxmax','real',
     +		'acceptance: max |theta_x| [D: pi/2]','1.5708')
	call addarg('-thymin','real',
     +		'acceptance: min |theta_y| [D: 0]','0')
	call addarg('-thymax','real',
     +		'acceptance: max |theta_y| [D: pi/2]','1.5708')

	return
	end

!=====================================================================

	integer function user_init ()

	implicit none
#include "frame.inc"

	integer iok
	integer lnblnk

! Declare routines for retrieval of command-line arguments

	integer		getarg_bool, getarg_int
	real		getarg_real
	character*80	getarg_char

! Initialize return code to success

	user_init = 0

! Retrieve any command line arguments that were declared

	file_base = 'nt'
	file_base = getarg_char('-out')
	nt_runsplit = ( getarg_bool('-runsplit') .gt. 0 )
	if (nt_runsplit) then
	  nt_split = .false.
	else
	  nt_split = ( getarg_bool('-split') .gt. 0 )
	  nt_size = -1
	  if ( nt_split ) nt_size = getarg_int('-nsplit') 
	endif
	req_scat = ( getarg_bool('-scat') .gt. 0 )
	req_accep = ( getarg_bool('-accep') .gt. 0 )
	req_lund = ( getarg_bool('-lund') .gt. 0 )
	req_charge = ( getarg_bool('-charge') .gt. 0 )
	req_photon = ( getarg_bool('-allneut') .eq. 0 )
	accep_pmin = getarg_real('-pmin')
	accep_thmin = getarg_real('-thmin')
	accep_thmax = getarg_real('-thmax')
	accep_thxmax = getarg_real('-thxmax')
	accep_thymin = getarg_real('-thymin')
	accep_thymax = getarg_real('-thymax')

! Clear job-level counters 

	c_nev = 0
	c_nevgen = 0
	c_ngoodev = 0
	ifile = 0
	nt_num = 0
	nt_icycle = 999999

! Open and book new ntuple (unless we are splitting at runs)

	if (.not.nt_runsplit) call book_ntuple

	return
	end

!=====================================================================

	integer function user_runinit ( runnum )

	implicit none
#include "frame.inc"

	integer runnum, i

! Initialize return code to success

	user_runinit = 0

! Reset run-level counters

! NOTE: runnum is always 0 when reading Monte Carlo GAF files
	run = runnum
	cr_nev = 0
	cr_nevgen = 0
	cr_ngoodev = 0

! Speak to the human

	ifile = ifile + 1
!	write(6,'(/a,i3,a)') 'Processing input file #', ifile, ' ...'
	write(6,'(/a,i3,a,i5)') 'Processing input file #', ifile, 
     +		' ... run number = ', runnum

! If we're splitting the ntuple every run, start a new ntuple file

	if (nt_runsplit) call book_ntuple

	return
	end

!=====================================================================

	integer function user_event ( runnum, eventnum, 
     +		eventtime, eventtimemu, index, cur1, cur2 )

	implicit none
#include "frame.inc"

	integer runnum, eventnum, eventtime, eventtimemu
	integer	index, cur1, cur2
	logical good_event

! Initialize return code to success

	user_event = 0

! Increment job-level and run-level event counters

	c_nev = c_nev + 1
	cr_nev = cr_nev + 1

! Fill event-level and track-level ntuple blocks

	call fill_event
	call fill_track
	call finish_event

! Some more counters to increment, for the number of generated events

	c_nevgen = c_nevgen + m_ievgen
	cr_nevgen = cr_nevgen + m_ievgen
	  
! Is this a good event? If so, store it in the ntuple

	if (good_event()) then

! ... udpate good event counters
	  c_ngoodev = c_ngoodev + 1
	  cr_ngoodev = cr_ngoodev + 1

! ... before storing, check if it's time to start a new ntuple file
	  if (nt_split .and. (c_ngoodev.gt.1)) then
	    if (mod(c_ngoodev,nt_size).eq.1) call book_ntuple
	  endif

! ... store the event
	  call hfnt (nt1)

	endif

! Keep the human informed

	if (mod(cr_nev,500).eq.0) 
     +		print *, 'processed ', cr_nev, ' events'

	return
	end

!=====================================================================

	integer function user_runend ( retcode )

	implicit none
#include "frame.inc"

	integer i, lun, retcode

! Initialize return code to whatever was handed to us

	user_runend = retcode

! Produce run-level counter report

	lun = 6
	write(lun,900) 
!	write(lun,901) 'RUN: ', run
	write(lun,901) 'FILE: ', ifile
	write(lun,901) 'Number of events processed = ', cr_nev
	write(lun,901) 'Number of generated events = ', cr_nevgen
	write(lun,901) 'Number of good events      = ', cr_ngoodev
	write(lun,900) 

900	format (1x,60('-'))
901	format (1x,a,i6)

! No problems

	user_runend = 0
	retcode = 0

	return
	end

!=====================================================================

	integer function user_end ( retcode )

	implicit none
#include "frame.inc"

	integer i, j, n, lun, retcode
	integer lnblnk

! Initialize return code to whatever was handed to us

	user_end = retcode

! Job-level counter report

	lun = 6
	write(lun,900) 
	write(lun,901) 'SUMMARY: ALL RUNS PROCESSED'
	write(lun,901) 'Number of events processed = ', c_nev
	write(lun,901) 'Number of generated events = ', c_nevgen
	write(lun,901) 'Number of good events      = ', c_ngoodev
	write(lun,900) 

900	format (/1x,60('='))
901	format (1x,a,i6)

! Save ntuple and close file

	call hrout (0, nt_icycle,' ')
	call hrend (ctop)
	close (nt_lun)
	write(6,'(2a)') '  >>> closed ntuple file: ', 
     +		nt_file(1:lnblnk(nt_file))

! No problems

	user_end = 0
	retcode = 0

	return
	end

!=====================================================================
! We don't use these user functions, just need empty templates

	integer function user_burstinit ( runnum, burstnum, 
     +		bursttime, bursttimemu, burstgood )

	implicit none
	integer runnum, burstnum, bursttime, bursttimemu, burstgood

! Initialize return code to success

	user_burstinit = 0

	return
	end

!---------------------------------------------------------------------

	integer function user_burstend ( retcode )

	implicit none
	integer retcode

! Initialize return code to success

	user_burstend = retcode

	return
	end
	
!---------------------------------------------------------------------

	integer function user_slow ( name, starttime, starttimemu,
     +		endtime, endtimemu, flag_first, status )

	implicit none
	character*(*)	name
	integer		starttime, starttimemu, endtime, endtimemu
	integer		flag_first, status

! Initialize return code to success

	user_slow = 0

	return
	end


!=====================================================================
! SECTION TWO: Following are codes specific to this program, 
!		e.g. to book and fill the ntuple, and perform calculations
!=====================================================================

!=====================================================================
! Open a new ntuple file and book the ntuple

	subroutine book_ntuple

	implicit none
#include "frame.inc"

	integer i, iok
	integer lnblnk
	character*80 string

! Update the ntuple file counter

	nt_num = nt_num + 1

! Construct the output filename from the user-specified filename base

	i = lnblnk(file_base)

! ... if we're splitting at every run, use base+runnumber
	if (nt_runsplit) then
	  write(nt_file,'(a,i5.5,a)') file_base(1:i), run, '.rz'

! ... if we're splitting every n records, update the ntuple counter 
! ... and use it in the filename
	else if (nt_split) then
	  write(string,'(i6)') nt_num
	  call cleft(string,1,80)
	  write(nt_file,99) file_base(1:i), string(1:lnblnk(string))
99	  format(a,'_',a,'.rz')

! ... if we are not doing any splitting of the ntuple, use the base directly
	else 
	  write(nt_file,'(2a)') file_base(1:i), '.rz'

	endif

! Open ntuple file

! ... if this is not the first file, we have to save and close the previous one
	if (nt_num .gt. 1) then
	  call hrout (0, nt_icycle,' ')
	  call hrend (ctop)
	  close (nt_lun)
	  write(6,'(2a)') '  >>> closed ntuple file: ', 
     +		nt_file(1:lnblnk(nt_file))
	endif

! ... open the new file
        call hropen (nt_lun,ctop,nt_file,'n',lrecl,iok)
        if (iok .ne. 0) then
	  print *, 'ERROR: Could not open output rz file.'
	  stop
	endif
	write(6,'(/2a)') '  >>> opened new ntuple file: ', 
     +		nt_file(1:lnblnk(nt_file))

! Book ntuple

	call hbset ('bsize',lrecl,iok)
	call hbnt (nt1,nt1_title,' ')

! Book each block of the big ntuple
! Arguments to hbname:
!	1 - ntuple index
!	2 - string denoting common block name (CHA8)
!	    (note: blocks using an index variable from another block 
!	    must have the same block name)
!	3 - first variable in this block
!	4 - string describing contents of block

! ... event-level blocks, all in common block /m_Event/
	call hbname (nt1,'m_Event',m_iEvent,cm_mcEvent)
	call hbname (nt1,'m_Event',m_Accep,cm_Event)
	call hbnamc (nt1,'m_Event',m_cType,cm_EventC)
	call hbname (nt1,'m_Event',mB_XVx,cm_mcBeam)

! ... track-level blocks, all in common block /mT_Track/
	call hbname (nt1,'mT_Track',mT_index,cmT_mcTrack)
	call hbname (nt1,'mT_Track',mT_istart,cmT_mcVert)
	call hbname (nt1,'mT_Track',mT_Accep,cmT_Track)
	call hbnamc (nt1,'mT_Track',mT_cType,cmT_TrackC)

	return
	end

!=====================================================================
! Fill event-level ntuple blocks

	subroutine fill_event 

	implicit none
#include "frame.inc"

! One sanity check

	if (coutab(mcEvent).ne.1) then
	  print *, '>>> mcEvent has ', coutab(mcEvent), ' rows ',
     +		' (event #', cr_nev, ')'
	  return
	endif

! Retrieve the table information

	mcEvent_ID = 1
	call gettab (mcEvent)
	mcBeam_ID = 1
	call gettab (mcBeam)

! Shuffle all interesting values to ntuple variables

! ... for the mcEVent table
	m_iEvent = mcEvent_iEvent
	m_iEvGen = mcEvent_iEvGen
	m_process = mcEvent_process
	m_nucleon = mcEvent_nucleon
	m_qflavour = mcEvent_qflavour
	m_remnant = mcEvent_remnant
	m_qtrack = mcEvent_qtrack
	m_ctype = mcEvent_ctype
	m_weight = mcEvent_weight
	m_x = mcEvent_x
	m_y = mcEvent_y
	m_nu = mcEvent_nu
	m_q2 = mcEvent_q2
	m_w2 = mcEvent_w2
	m_ep = mcEvent_ep
	m_the = mcEvent_the
	m_phi = mcEvent_phi
	m_polb = mcEvent_polb
	m_polt = mcEvent_polt

! ... same for the mcBeam table
	mB_XVx = mcBeam_XVx
	mB_YVx = mcBeam_YVx
	mB_ZVx = mcBeam_ZVx
	mB_Ebeam = mcBeam_Ebe

! Set defaults for the stuff we will calculate in `finish_event'
! from TRACK-level information

	m_accep = 0
	m_iScat = 0
	m_thx = 0
	m_thy = 0

	return
	end

!=====================================================================
! Fill track-level ntuple blocks

	subroutine fill_track

	implicit none
#include "frame.inc"

	integer	MAXVERT
	parameter (MAXVERT=200)
	real	xvx(MAXVERT), yvx(MAXVERT), zvx(MAXVERT)

	integer i, j, it, itfull, itparent, ntfull, iok, ix, iy, ith
	integer	getbit
	integer full_iltype(MT_MAX), full_parent(MT_MAX)
	integer	full_accep(MT_MAX), full_it(MT_MAX)
	real	full_m(MT_MAX)
	integer	child_ig(20)
	real	ax, ay, az, r, rx, ry
	logical good_track, check_accep

! Store all space points from mcVert into a local table, once and for all

	if (coutab(mcVert).gt.MAXVERT)  then
	  print *, 'ERROR: mcVert has ', coutab(mcVert), ' rows'
	  stop 'Increase MAXVERT in routine FILL_TRACK'
	endif
	
	do mcVert_ID = 1, coutab(mcVert)
	  call gettab(mcVert)
	  xvx(mcVert_ID) = mcVert_X
	  yvx(mcVert_ID) = mcVert_Y
	  zvx(mcVert_ID) = mcVert_Z
	enddo

! Check number of mcTrack rows against array dimension

	ntfull = coutab (mcTrack)
	if (ntfull.gt.MT_MAX) then
	  print *, '>>> number of tracks truncated from ', ntfull, 
     +		' to ', MT_MAX
	  print *, '>>> ievent = ', m_ievent
	  ntfull = MT_MAX
	endif

! ---------- START LOOP OVER TRACKS ---------------------------------

	it = 0
	do itfull = 1, ntfull

! Retrieve the table information for this row

	  it = it + 1
	  mcTrack_ID = itfull
	  call gettab(mcTrack)

! Store some info for ALL tracks in a local array, for parent lookup

	  full_parent(itfull) = mcTrack_Parent
	  if (mcTrack_Parent .eq. INULL) full_parent(itfull) = 0
	  full_iltype(itfull) = mcTrack_iLType
	  full_m(itfull) = mcTrack_m
	  full_accep(itfull) = -1
	  full_it(itfull) = it
! ........ sanity check: our algorithm requires ordered parent links
	  if (full_parent(itfull).ge.itfull) then
	    print *, "ERROR! Track ID of Parent >= Child: ", 
     +		full_parent(itfull), " >= ", itfull
	    stop
	  endif

! Shuffle all interesting values from mcTrack to ntuple variables

	  mT_iCharge(it)	= mcTrack_iCharge
	  mT_iGType(it)		= mcTrack_iGType
	  mT_iLType(it)		= mcTrack_iLType
	  mT_Parent(it)		= mcTrack_Parent
	  if (mcTrack_Parent .eq. INULL) mt_Parent(it) = 0
	  mT_cType(it)		= mcTrack_cType
	  mT_The(it)		= mcTrack_The
	  mT_Phi(it)		= mcTrack_Phi
	  mT_Px(it)		= mcTrack_Px
	  mT_Py(it)		= mcTrack_Py
	  mT_Pz(it)		= mcTrack_Pz
	  mT_P(it)		= mcTrack_P
	  mT_E(it)		= mcTrack_E
	  mT_M(it)		= mcTrack_M

! Shuffle all interesting values from mcVert to ntuple variables

	  if (mcTrack_startvert.eq.INULL) then
	    mT_istart(it)	= 0
	    mT_startX(it)	= 99999.
	    mT_startY(it)	= 99999.
	    mT_startZ(it)	= 99999.
	  else
	    mT_istart(it)	= mcTrack_startvert
	    mT_startX(it)	= xvx(mcTrack_startvert)
	    mT_startY(it)	= yvx(mcTrack_startvert)
	    mT_startZ(it)	= zvx(mcTrack_startvert)
	  endif
	  if (mcTrack_stopvert.eq.INULL) then
	    mT_istop(it)	= 0
	    mT_stopX(it)	= 99999.
	    mT_stopY(it)	= 99999.
	    mT_stopZ(it)	= 99999.
	  else
	    mT_istop(it)	= mcTrack_stopvert
	    mT_stopX(it)	= xvx(mcTrack_stopvert)
	    mT_stopY(it)	= yvx(mcTrack_stopvert)
	    mT_stopZ(it)	= zvx(mcTrack_stopvert)
	  endif
	
! Calculate the thx and thy planar angles, and store z value

	  ax = sin(mcTrack_The)*cos(mcTrack_Phi)
	  ay = sin(mcTrack_The)*sin(mcTrack_Phi)
	  az = cos(mcTrack_The)
	  mT_thx(it) = 0.
	  if (ax.ne.0.and.az.ne.0) mT_thx(it) = atan2(ax,az)
	  mT_thy(it) = 0.
	  if (ay.ne.0.and.az.ne.0) mT_thy(it) = atan2(ay,az)
	  mT_z(it) = 0.
	  if (m_nu.gt.0.) mt_z(it) = mt_E(it)/m_nu

! Determine Lund type of particle's parent

	  mT_iLparent(it) = 0
	  mT_Mparent(it) = -1
	  if (mT_parent(it).gt.0) then
	    if (mT_parent(it).ge.itfull) then
	      write(6,'(a,i4,a,i4)') 'ERROR: Track # ', itfull, 
     +		' has parent track # ', mT_parent(it)
	    else
	      mT_iLparent(it) = full_iLtype(mT_parent(it))
	      mT_Mparent(it) = full_m(mT_parent(it))
	    endif
	  endif

! Check for acceptance

! ... check whether track was accepted
	  mT_accep(it) = -1
	  if (check_accep) mT_accep(it) = 1

! ... store result in 'full' array
	  full_accep(itfull) = mT_accep(it)

! If this is the scattered beam track, store its index for processing
! by the `finish_event' routine.

	  if (mT_cType(it).eq.'scat' .or.
     +		mt_cType(it).eq.'back') m_iScat = it

! Do we want to keep this track?

	  if (.not.good_track(it)) then
	    it = it - 1
	    full_it(itfull) = 0
	  endif

! ---------- END LOOP OVER TRACKS ---------------------------------

	enddo ! <loop over tracks>

! Update mT_index to reflect the actual number of tracks stored

	mT_index = it

! Check acceptance of unstable particles --> were all child particles accepted?

	do itfull = ntfull, 1, -1
	  if (full_accep(itfull).ge.0) then
	    itparent = full_parent(itfull)
	    if (full_accep(itparent).lt.0) then
		full_accep(itparent) = full_accep(itfull)
	    else if (full_accep(itfull)*full_accep(itparent).eq.0) then
		full_accep(itparent) = 0
	    else
		full_accep(itparent) = full_accep(itparent) +
     +			full_accep(itfull)
	    endif
	  endif
	enddo

	do itfull = 1, ntfull
	  it = full_it(itfull)
! ........ to avoid confusion: remove 'init' particles from consideration
	  if (it.gt.0) then
	    if (mt_ctype(it).eq.'init') then
	      mt_accep2(it) = -1
	    else
	      mt_accep2(it) = full_accep(itfull)
	    endif
	  endif
	enddo
	  
	return
	end

!=====================================================================
! Check whether track presently in the mcTrack WCB is within acceptance

	logical function check_accep()

	implicit none
#include "frame.inc"

	real	thx, thy

	check_accep = .false.
	if (mcTrack_P.lt.accep_pmin) return
	if (mcTrack_The.lt.accep_thmin) return
	if (mcTrack_The.gt.accep_thmax) return
	thx = 0.
	if (mcTrack_Px.ne.0..and.mcTrack_Pz.ne.0.) 
     +		thx = atan2(mcTrack_Px,mcTrack_Pz)
	if (abs(thx).gt.accep_thxmax) return
	thy = 0.
	if (mcTrack_Py.ne.0..and.mcTrack_Pz.ne.0.) 
     +		thy = atan2(mcTrack_Py,mcTrack_Pz)
	if (abs(thy).gt.accep_thymax) return
	if (abs(thy).lt.accep_thymin) return
	check_accep = .true.

	return
	end

!=====================================================================
! Fill remaining event-level variables with info computed at track level

	subroutine finish_event

	implicit none
#include "frame.inc"

! Fill a few more event-level entries concerning the scattered beam track

	if (m_iScat.gt.0) then
	  m_accep = mT_accep(m_iScat)
	  m_thx = mT_thx(m_iScat)
	  m_thy = mT_thy(m_iScat)
	endif

	return
	end

!=====================================================================
! Determine what is a good event --> one worth storing in the ntuple

	logical function good_event ()

	implicit none
#include "frame.inc"

! Here, you can place criteria on the event ... if the function returns
! false, the event will not be stored in the ntuple.

	good_event = .true.

! ... require that scattered positron be accepted
	if (req_accep) then
	  if (m_Accep.le.0) good_event = .false.
	endif

	return
	end

!=====================================================================
! Determine what is a good track --> one worth storing in the ntuple

	logical function good_track ( it )

	implicit none
#include "frame.inc"

	integer it

! Here, you can place criteria on individual tracks ... if the function returns
! false for given track 'it', that track will not be stored in the ntuple.

	good_track = .true.

! ... require accepted tracks
	if (req_accep) then
! GNOME: switch permanently to accep2
!	  if (mT_Accep(it).le.0) good_track = .false.
	  if (mT_Accep2(it).le.0) good_track = .false.
	endif

! ... require lund type tracks
	if (req_lund) then
	  if (mT_cType(it).ne.'scat'.and.mt_cType(it).ne.'lund'
     +		.and.mT_cType(it).ne.'back') good_track = .false.
	endif

! ... require charged tracks
	if (req_charge) then
	  if (mT_iCharge(it).eq.0) good_track = .false.
	endif

	return
	end

!=====================================================================
! Utility function which returns the value of bit # <bit> in word.
! Note that bit numbers start at 1, not 0.

	integer function getbit ( inword, bit )

	implicit none
	integer	inword, bit, word

! Watch for negative values in signed integers! 

	word = inword
	if (word.lt.0) then
	  if (bit.eq.32) then
	    getbit = 1
	    return
	  endif
	  word = word - 2**31
	endif

	getbit = mod( int(word/2**(bit-1)) , 2 )
	if (getbit.lt.0) 
     +	  print *, '>>> getbit!!! got negative value from bit ',
     +		bit, ' of word ', inword

	return
	end

