
! ============ ADAMO INCLUDE FILES ============

#include "partap.inc"

! ADAMO tables we will access

#include "mcTrack.inc"
#include "mcEvent.inc"
#include "mcBeam.inc"
#include "mcVert.inc"

! ============ GENERAL VARIABLES ============

! Some useful constants

	real	pi
	parameter (pi = 3.14159)

! General HBOOK parameters describing our ntuple

	integer nt_lun,            ! LUN for ntuple rz file 
     +		nt1,               ! ntuple id 1
     +          lrecl              ! record length for rz files
	character ctop*8,          ! top directory name
     +          nt1_title*8	   ! title for ntuple 1
      
	parameter (nt_lun=50)
	parameter (nt1=1)
	parameter (lrecl=1024)
	parameter (ctop='USERCODE')
	parameter (nt1_title='TheNtuple')

! User switches 

	character	file_base*80,
     +			nt_file*80
	integer		nt_size
	real		accep_pmin,
     +			accep_thmin,
     +			accep_thmax,
     +			accep_thxmax,
     +			accep_thymin,
     +			accep_thymax
	logical		nt_split,
     +			nt_runsplit,
     +			req_scat,
     +			req_accep,
     +			req_lund,
     +			req_charge,
     +			req_photon

	common /switches/ 
     +		file_base,
     +		nt_file,
     +		nt_size,
     +		accep_pmin,
     +		accep_thmin,
     +		accep_thmax,
     +		accep_thxmax,
     +		accep_thymin,
     +		accep_thymax,
     +		nt_split,
     +		nt_runsplit,
     +		req_scat,
     +		req_accep,
     +		req_lund,
     +		req_charge,
     +		req_photon

! Counters (n_ denotes job-level counters, nr_ are cleared at every run)

	integer	run, ifile, nt_num, nt_icycle,
     +		c_nev, c_nevgen, c_ngoodev, 
     +		cr_nev, cr_nevgen, cr_ngoodev 
	common /counters/
     +		run, ifile, nt_num, nt_icycle,
     +		c_nev, c_nevgen, c_ngoodev, 
     +		cr_nev, cr_nevgen, cr_ngoodev

! =========== NTUPLE BLOCKS ============

! Components of names
!	- Let 'table' be the table name from which the ntuple entries come
!	  ... if they are user-calculated, a simply non-table name is used
!	- Here, the overall 'letter' is 'r', denoting HRC-type information
!	- For array-type blocks, add another letter: 'rT' for track blocks
! For all blocks:
!	- common block name: r_table
!	- string specifier for block: cr_table
! For event-level blocks:
!	- entries: r_entry
! For array-type blocks:
!	- entries: rT_entry
!	- array index variable: rT_index
!	- integer parameter for max entries: RT_MAX
!	- character version of that: CRT_MAX
! The common blocks:
!	- all blocks indexed the same way go the same commmon: r_Track

! Event-level ntuple blocks, all in common block /m_Event/

! ... from table: mcEvent
	character cm_mcEvent*200
	parameter (cm_mcEvent=
     +		'm_iEvent:I,'//
     +		'm_iEvGen:I,'//
     +		'm_process:I,'//
     +		'm_nucleon:I,'//
     +		'm_qflavour:I,'//
     +		'm_remnant:I,'//
     +		'm_qTrack:I,'//
     +		'm_weight:R,'//
     +		'm_x:R,'//
     +		'm_y:R,'//
     +		'm_nu:R,'//
     +		'm_q2:R,'//
     +		'm_w2:R,'//
     +		'm_ep:R,'//
     +		'm_the:R,'//
     +		'm_phi:R,'//
     +		'm_polb:R,'//
     +		'm_polt:R')
	integer	m_iEvent,
     +		m_iEvGen,
     +		m_process,
     +		m_nucleon,
     +		m_qflavour,
     +		m_remnant,
     +		m_qTrack
	real	m_weight,
     +		m_x,
     +		m_y,
     +		m_nu,
     +		m_q2,
     +		m_w2,
     +		m_ep,
     +		m_the,
     +		m_phi,
     +		m_polb,
     +		m_polt

! ... stuff we're gonna calculate
	character cm_Event*200
	parameter (cm_Event=
     +		'm_Accep:I,'//
     +		'm_iScat:I,'//
     +		'm_thx:R,'//
     +		'm_thy:R')
	integer m_Accep,
     +		m_iScat
	real	m_thx,
     +		m_thy

! ... character entries must be in their own block
	character cm_EventC*100
	parameter (cm_EventC=
     +		'm_cType:C*4')
	character*4 m_cType

! ... from table: mcBeam
	character cm_mcBeam*200
	parameter (cm_mcBeam=
     +		'mB_XVx:R,'//
     +		'mB_YVx:R,'//
     +		'mB_ZVx:R,'//
     +		'mB_Ebeam:R')
	real	mB_XVx,
     +		mB_YVx,
     +		mB_ZVx,
     +		mB_Ebeam

! ... everyone together in the common block
	common	/m_Event/
     +		m_iEvent,
     +		m_iEvGen,
     +		m_process,
     +		m_nucleon,
     +		m_qflavour,
     +		m_remnant,
     +		m_qTrack,
     +		m_weight,
     +		m_x,
     +		m_y,
     +		m_nu,
     +		m_q2,
     +		m_w2,
     +		m_ep,
     +		m_the,
     +		m_phi,
     +		m_polb,
     +		m_polt,
     +		m_Accep,
     +		m_iScat,
     +		m_thx,
     +		m_thy,
     +		m_cType,
     +		mB_XVx,
     +		mB_YVx,
     +		mB_ZVx,
     +		mB_Ebeam

! Track-level ntuple blocks, all in common block /mT_Track/

! ... first set the max number of entries at track-level
	integer		MT_MAX
	character*3	CMT_MAX
	parameter (MT_MAX=200)
	parameter (CMT_MAX='200')

! ... from table: mcTrack
	character cmT_mcTrack*500
	parameter (cmT_mcTrack= 
     +		'mT_index[0,'//CMT_MAX//']:I,'//
     +		'mT_iCharge(mT_index):I,'//
     +		'mT_iGType(mT_index):I,'//
     +		'mT_iLType(mT_index):I,'//
     +		'mT_Parent(mT_index):I,'//
     +		'mT_The(mT_index):R,'//
     +		'mT_phi(mT_index):R,'//
     +		'mT_px(mT_index):R,'//
     +		'mT_py(mT_index):R,'//
     +		'mT_pz(mT_index):R,'//
     +		'mT_p(mT_index):R,'//
     +		'mT_e(mT_index):R,'//
     +		'mT_m(mT_index):R')
     	integer	mT_index,
     +		mT_iCharge(MT_MAX),
     +		mT_iGType(MT_MAX),
     +		mT_iLType(MT_MAX),
     +		mT_Parent(MT_MAX)
	real	mT_The(MT_MAX),
     +		mT_phi(MT_MAX),
     +		mT_px(MT_MAX),
     +		mT_py(MT_MAX),
     +		mT_pz(MT_MAX),
     +		mT_p(MT_MAX),
     +		mT_e(MT_MAX),
     +		mT_m(MT_MAX)
	
! ... from table: mcVert
	character cmT_mcVert*300
	parameter (cmT_mcVert= 
     +		'mT_istart(mT_index):I,'//
     +		'mT_istop(mT_index):I,'//
     +		'mT_startX(mT_index):R,'//
     +		'mT_startY(mT_index):R,'//
     +		'mT_startZ(mT_index):R,'//
     +		'mT_stopX(mT_index):R,'//
     +		'mT_stopY(mT_index):R,'//
     +		'mT_stopZ(mT_index):R')
	integer mT_istart(MT_MAX),
     +		mT_istop(MT_MAX)
	real	mT_startX(MT_MAX),
     +		mT_startY(MT_MAX),
     +		mT_startZ(MT_MAX),
     +		mT_stopX(MT_MAX),
     +		mT_stopY(MT_MAX),
     +		mT_stopZ(MT_MAX)

! ... a few computed things
	character cmT_Track*200
	parameter (cmT_Track= 
     +		'mT_Accep(mT_index):I,'//
     +		'mT_Accep2(mT_index):I,'//
     +		'mT_iLParent(mT_index):I,'//
     +		'mT_MParent(mT_index):R,'//
     +		'mT_z(mT_index):R,'//
     +		'mT_Thx(mT_index):R,'//
     +		'mT_Thy(mT_index):R')
	integer mT_Accep(MT_MAX),
     +		mT_Accep2(MT_MAX),
     +		mT_iLParent(MT_MAX)
	real	mT_MParent(MT_MAX),
     +		mT_z(MT_MAX),
     +		mT_Thx(MT_MAX),
     +		mT_Thy(MT_MAX)

! ... character entries must be in their own block
	character cmT_TrackC*100
	parameter (cmT_TrackC=
     +		'mT_cType(mT_index):C*4')
	character*4 mT_cType(MT_MAX)

! ... everyone together in the common block
	common /mT_Track/
     +		mT_index,
     +		mT_iCharge,
     +		mT_iGType,
     +		mT_iLType,
     +		mT_Parent,
     +		mT_The,
     +		mT_phi,
     +		mT_px,
     +		mT_py,
     +		mT_pz,
     +		mT_p,
     +		mT_e,
     +		mT_m,
     +		mT_istart,
     +		mT_istop,
     +		mT_startX,
     +		mT_startY,
     +		mT_startZ,
     +		mT_stopX,
     +		mT_stopY,
     +		mT_stopZ,
     +		mT_Accep,
     +		mT_Accep2,
     +		mT_iLParent,
     +		mT_MParent,
     +		mT_z,
     +		mT_Thx,
     +		mT_Thy,
     +		mT_cType

